/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.elements;

import java.util.List;

import org.dom4j.Attribute;
import org.dom4j.Element;
import org.dom4j.Node;

/**
 * @author Kenji Heigel
 */
public class ConditionPoshiElement extends ExecutePoshiElement {

	@Override
	public PoshiElement clone(Element element) {
		if (isElementType(_ELEMENT_NAME, element)) {
			return new ConditionPoshiElement(element);
		}

		return null;
	}

	@Override
	public PoshiElement clone(
		PoshiElement parentPoshiElement, String readableSyntax) {

		if (_isElementType(parentPoshiElement, readableSyntax)) {
			return new ConditionPoshiElement(
				parentPoshiElement, readableSyntax);
		}

		return null;
	}

	protected ConditionPoshiElement() {
	}

	protected ConditionPoshiElement(Element element) {
		super(_ELEMENT_NAME, element);
	}

	protected ConditionPoshiElement(
		List<Attribute> attributes, List<Node> nodes) {

		super(_ELEMENT_NAME, attributes, nodes);
	}

	protected ConditionPoshiElement(
		PoshiElement parentPoshiElement, String readableSyntax) {

		super(_ELEMENT_NAME, parentPoshiElement, readableSyntax);
	}

	@Override
	protected String createFunctionReadableBlock(String content) {
		String readableBlock = super.createFunctionReadableBlock(content);

		readableBlock = readableBlock.trim();

		if (readableBlock.endsWith(";")) {
			readableBlock = readableBlock.substring(
				0, readableBlock.length() - 1);
		}

		return readableBlock;
	}

	@Override
	protected String getBlockName() {
		return attributeValue("function");
	}

	private boolean _isElementType(
		PoshiElement parentPoshiElement, String readableSyntax) {

		if (!isConditionValidInParent(parentPoshiElement)) {
			return false;
		}

		if (readableSyntax.contains(" && ") ||
			readableSyntax.contains(" || ") || readableSyntax.startsWith("!") ||
			readableSyntax.startsWith("else if (")) {

			return false;
		}

		if (readableSyntax.endsWith(")") &&
			!readableSyntax.startsWith("isSet(")) {

			return true;
		}

		return false;
	}

	private static final String _ELEMENT_NAME = "condition";

}