/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.elements;

import com.liferay.poshi.runner.util.StringUtil;

import org.dom4j.Comment;

/**
 * @author Peter Yoo
 */
public class InlinePoshiComment extends PoshiComment {

	@Override
	public PoshiComment clone(Comment comment) {
		String commentText = comment.getText();

		if (!commentText.contains("\n")) {
			return new InlinePoshiComment(comment);
		}

		return null;
	}

	@Override
	public PoshiComment clone(String readableSyntax) {
		if (isReadableSyntaxComment(readableSyntax)) {
			return new InlinePoshiComment(readableSyntax);
		}

		return null;
	}

	@Override
	public boolean isReadableSyntaxComment(String readableSyntax) {
		if (readableSyntax.startsWith("//")) {
			return true;
		}

		return false;
	}

	@Override
	public void parseReadableSyntax(String readableSyntax) {
		if (isReadableSyntaxComment(readableSyntax)) {
			String text = readableSyntax.substring(2);

			setText(" " + text.trim() + " ");
		}
	}

	@Override
	public String toReadableSyntax() {
		StringBuilder sb = new StringBuilder();

		sb.append("\n\t//");
		sb.append(StringUtil.trimTrailing(getText()));

		return sb.toString();
	}

	protected InlinePoshiComment() {
	}

	protected InlinePoshiComment(Comment comment) {
		super(comment);
	}

	protected InlinePoshiComment(String readableSyntax) {
		super(readableSyntax);
	}

}