/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.elements;

import org.dom4j.Comment;

/**
 * @author Kenji Heigel
 */
public class MultilinePoshiComment extends PoshiComment {

	@Override
	public PoshiComment clone(Comment comment) {
		String commentText = comment.getText();

		if (commentText.contains("\n")) {
			return new MultilinePoshiComment(comment);
		}

		return null;
	}

	@Override
	public PoshiComment clone(String readableSyntax) {
		if (isReadableSyntaxComment(readableSyntax)) {
			return new MultilinePoshiComment(readableSyntax);
		}

		return null;
	}

	@Override
	public boolean isReadableSyntaxComment(String readableSyntax) {
		if (readableSyntax.endsWith("*/") && readableSyntax.startsWith("/*")) {
			return true;
		}

		return false;
	}

	@Override
	public void parseReadableSyntax(String readableSyntax) {
		String text = readableSyntax.substring(2, readableSyntax.length() - 2);

		setText(text);
	}

	@Override
	public String toReadableSyntax() {
		StringBuilder sb = new StringBuilder();

		sb.append("\n\t/*");
		sb.append(getText());
		sb.append("*/");

		return sb.toString();
	}

	protected MultilinePoshiComment() {
	}

	protected MultilinePoshiComment(Comment comment) {
		super(comment);
	}

	protected MultilinePoshiComment(String readableSyntax) {
		super(readableSyntax);
	}

}