/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.prose;

import com.liferay.poshi.runner.util.Dom4JUtil;
import com.liferay.poshi.runner.util.StringUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.dom4j.Element;
import org.dom4j.tree.DefaultAttribute;

/**
 * @author Yi-Chen Tsai
 */
public class PoshiProseScenario {

	public PoshiProseScenario(String poshiProseScenarioString) {
		Matcher matcher = _scenarioPattern.matcher(poshiProseScenarioString);

		if (!matcher.find()) {
			throw new RuntimeException(
				"Prose scenario does not match pattern " +
					_scenarioPattern.pattern() + "\n" +
						poshiProseScenarioString);
		}

		_scenarioName = matcher.group("name");
		_scenarioContent = matcher.group("content");

		List<String> poshiProseStatementStrings = StringUtil.split(
			_scenarioContent, PoshiProseStatement.KEYWORDS);

		for (String poshiProseStatementString : poshiProseStatementStrings) {
			_poshiProseStatements.add(
				new PoshiProseStatement(poshiProseStatementString));
		}
	}

	public Element toElement() {
		Element commandElement = Dom4JUtil.getNewElement(
			"command", null, new DefaultAttribute("name", _scenarioName));

		for (PoshiProseStatement poshiProseStatement : _poshiProseStatements) {
			commandElement.add(poshiProseStatement.toElement());
		}

		return commandElement;
	}

	protected static final String[] KEYWORDS = {"Scenario"};

	private final List<PoshiProseStatement> _poshiProseStatements =
		new ArrayList<>();
	private final String _scenarioContent;
	private final String _scenarioName;
	private final Pattern _scenarioPattern = Pattern.compile(
		"Scenario:[\\s]*(?<name>\\w+)[^\\w]*(?<content>[\\s\\S]*)");

}